#include <stdio.h>
#include <math.h>
#include "SDL.h"
#include "bump.h"

int sdl_after_init = 0;
SDL_Surface *screen = NULL;

void sdl_error(char *msg) {
  fprintf(stderr, "%s: %s\n", msg, SDL_GetError());
}

void cleanup_sdl(void) {
  if (sdl_after_init)
    SDL_Quit();
}

int init_sdl(void) {
  if (SDL_Init(SDL_INIT_VIDEO) < 0) {
    sdl_error("SDL_Init");
    return -1;
  }
  sdl_after_init = 1;

  screen = SDL_SetVideoMode(WIDTH, HEIGHT, 0, 0);
  if (!screen) {
    sdl_error("SDL_SetVideoMode");
    return -1;
  }

  return 0;
}

void print_format(SDL_PixelFormat *fmt) {
  printf("bits per pixel: %d\nbytes per pixel: %d\n",
	 fmt->BitsPerPixel, fmt->BytesPerPixel);
  printf("red mask   = %08x\n"
	 "green mask = %08x\n"
	 "blue mask  = %08x\n", fmt->Rmask, fmt->Gmask, fmt->Bmask);
}

/* wczytaj obrazek i przekszta bitmap na zadany format */

SDL_Surface *load_and_convert_bmp(char *name, SDL_PixelFormat *fmt) {
  SDL_Surface *bmp, *conv;
  bmp = SDL_LoadBMP(name);
  if (!bmp) {
    sdl_error("SDL_LoadBMP");
    return NULL;
  }
  conv = SDL_ConvertSurface(bmp, fmt, 0);
  SDL_FreeSurface(bmp);
  if (!conv)
    sdl_error("SDL_ConvertSurface");
  return conv;
}

int closed(void) {
  SDL_Event ev;
  while (SDL_PollEvent(&ev)) {
    if (ev.type == SDL_QUIT)
      return 1;
  }
  return 0;
}

SDL_Surface *create_working_surface(int w, int h) {
  SDL_Surface *s;
  /* stwrz bitmap o 32 bitach na piksel */
  /* skadowa czerwona zajmuje bity 23-16, zielona 15-8, niebieska 7-0 */
  s = SDL_CreateRGBSurface(0, w, h, 32,
			   0xff0000, 0x00ff00, 0x0000ff, 0);
  if (!s)
    sdl_error("SDL_CreateRGBSurface");
  return s;
}

SDL_Surface *create_map_surface(int w, int h) {
  SDL_Surface *s;
  SDL_Color colors[256];
  int i;

  s = SDL_CreateRGBSurface(0, w, h, 8, 0, 0, 0, 0);
  if (!s)
    sdl_error("SDL_CreateRGBSurface");

  for (i = 0; i < 256; i++)
    colors[i].r = colors[i].g = colors[i].b = i;
  SDL_SetPalette(s, SDL_LOGPAL, colors, 0, 256);

  return s;
}

void animation(SDL_Surface *map, SDL_Surface *texture, SDL_Surface *work) {
  int ang;
  int x1, x2, y1, y2;
  SDL_Rect rect[2], r2;

  init_bump();

  ang = 250;
  x1 = y1 = 0;
  while (!closed()) {
    double a = 2.0 * M_PI * ang / 360;

    rect[0].x = x1;
    rect[0].y = y1;
    rect[0].w = rect[0].h = LIGHT_SIZE;
    SDL_FillRect(screen, rect, SDL_MapRGB(screen->format, 0, 0, 0));

    x2 = map->w / 2 - LIGHT_SIZE / 2 + (map->w / 2 - LIGHT_SIZE / 2) * sin(a);
    y2 = map->h / 2 - LIGHT_SIZE / 2 + (map->h / 2 - LIGHT_SIZE / 2) * cos(a);
    bump(map, texture, work, x2, y2);
    rect[1].x = x2;
    rect[1].y = y2;
    rect[1].w = rect[1].h = LIGHT_SIZE;
    r2.x = r2.y = 0;
    r2.w = r2.h = LIGHT_SIZE;
    SDL_BlitSurface(work, &r2, screen, rect + 1);

    SDL_UpdateRects(screen, 2, rect);

    ang -= 3;
    if (ang < 0)
      ang = 359;

    x1 = x2;
    y1 = y2;
  }
}

static inline Uint32 get_pixel(SDL_Surface *s, int x, int y) {
  return ((Uint32 *)(s->pixels + y * s->pitch))[x];
}

void show(void) {
  SDL_Surface *work, *map;
  /* wczytaj obrazek */
  SDL_Surface *bmp = SDL_LoadBMP("bump.bmp");
  if (!bmp) {
    sdl_error("SDL_LoadBMP");
    return;
  }
  map = create_map_surface(bmp->w, bmp->h);
  work = create_working_surface(bmp->w, bmp->h);
  if (map && work) {
    int i, j;
    SDL_Surface *texture;

    SDL_BlitSurface(bmp, NULL, work, NULL);
    for (i = 0; i < work->h; i++)
      for (j = 0; j < work->w; j++)
	((Uint8 *)(map->pixels + map->pitch * i))[j] =
	  get_pixel(work, j, i) & 0xff;

    SDL_FreeSurface(bmp);
    texture = load_and_convert_bmp("texture.bmp", work->format);
    if (texture) {
      animation(map, texture, work);
      SDL_FreeSurface(texture);
    }
  }
  if (work)
    SDL_FreeSurface(work);
  if (map)
    SDL_FreeSurface(map);
}

int main(void) {
  if (init_sdl() >= 0)
    show();
  cleanup_sdl();
  return 0;
}
